@@ -274,9 +274,9 @@ add_a_mime_part (GMimeMessage *message)
 	
 	/* create the content object - since the stream is not base64
 	   or QP encoded or anything, we can use
-	   GMIME_PART_ENCODING_DEFAULT as the encoding type (_DEFAULT
+	   GMIME_CONTENT_ENCODING_DEFAULT as the encoding type (_DEFAULT
 	   is the same as saying "nothing specified") */
-	content = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+	content = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
 	g_object_unref (stream);
 	
 	/* set the content object on the new mime part */
@@ -284,7 +284,7 @@ add_a_mime_part (GMimeMessage *message)
 	g_object_unref (content);
 	
 	/* if we want, we can tell GMime that the content should be base64 encoded when written to disk... */
-	g_mime_part_set_encoding (mime_part, GMIME_PART_ENCODING_BASE64);
+	g_mime_part_set_content_encoding (mime_part, GMIME_CONTENT_ENCODING_BASE64);
 	
 	/* the "polite" way to modify a mime structure that we didn't
 	   create is to create a new toplevel multipart/mixed part and
@@ -295,13 +295,15 @@ add_a_mime_part (GMimeMessage *message)
 	multipart = g_mime_multipart_new_with_subtype ("mixed");
 	
 	/* add our new text part to it */
-	g_mime_multipart_add_part (multipart, (GMimeObject *) mime_part);
+	g_mime_multipart_add (multipart, (GMimeObject *) mime_part);
+	g_object_unref (mime_part);
 	
 	/* now append the message's toplevel part to our multipart */
-	g_mime_multipart_add_part (multipart, message->mime_part);
+	g_mime_multipart_add (multipart, message->mime_part);
 	
 	/* now replace the message's toplevel mime part with our new multipart */
 	g_mime_message_set_mime_part (message, (GMimeObject *) multipart);
+	g_object_unref (multipart);
 }
 
 static void
@@ -317,7 +319,7 @@ remove_a_mime_part (GMimeMessage *message)
 	multipart = (GMimeMultipart *) message->mime_part;
 	
 	/* subpart indexes start at 0 */
-	g_mime_multipart_remove_part_at (multipart, 0);
+	g_mime_multipart_remove_at (multipart, 0);
 	
 	/* now we should be left with a toplevel multipart/mixed which
 	   contains the mime parts of the original message */
