@@ -207,7 +207,8 @@ cipher_sign (GMimeCipherContext *ctx, const char *userid, GMimeCipherHash hash,
  *
  * Signs the input stream and writes the resulting signature to the output stream.
  *
- * Returns: %0 on success or %-1 on fail.
+ * Returns: the #GMimeCipherHash used on success (useful if @hash is
+ * specified as #GMIME_CIPHER_HASH_DEFAULT) or %-1 on fail.
  **/
 int
 g_mime_cipher_context_sign (GMimeCipherContext *ctx, const char *userid, GMimeCipherHash hash,
@@ -526,7 +527,7 @@ g_mime_signer_set_errors (GMimeSigner *signer, GMimeSignerError errors)
  * Returns: the signer errors.
  **/
 GMimeSignerError
-g_mime_signer_get_error (GMimeSigner *signer)
+g_mime_signer_get_errors (GMimeSigner *signer)
 {
 	g_return_val_if_fail (signer != NULL, GMIME_SIGNER_ERROR_NONE);
 	
