@@ -192,11 +192,6 @@ g_mime_data_wrapper_get_stream (GMimeDataWrapper *wrapper)
 {
 	g_return_val_if_fail (GMIME_IS_DATA_WRAPPER (wrapper), NULL);
 	
-	if (wrapper->stream == NULL)
-		return NULL;
-	
-	g_object_ref (wrapper->stream);
-	
 	return wrapper->stream;
 }
 
@@ -248,7 +243,7 @@ write_to_stream (GMimeDataWrapper *wrapper, GMimeStream *stream)
 	case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
 	case GMIME_CONTENT_ENCODING_UUENCODE:
 		filter = g_mime_filter_basic_new (wrapper->encoding, FALSE);
-		filtered_stream = g_mime_stream_filter_new_with_stream (wrapper->stream);
+		filtered_stream = g_mime_stream_filter_new (wrapper->stream);
 		g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), filter);
 		g_object_unref (filter);
 		break;
