@@ -36,38 +36,23 @@ G_BEGIN_DECLS
 typedef struct _GMimeFilterCRLF GMimeFilterCRLF;
 typedef struct _GMimeFilterCRLFClass GMimeFilterCRLFClass;
 
-
 /**
- * GMimeFilterCRLFDirection:
- * @GMIME_FILTER_CRLF_ENCODE: Convert from Unix line endings to CRLF.
- * @GMIME_FILTER_CRLF_DECODE: Convert from CRLF to Unix line endings.
+ * GMimeFilterCRLF:
+ * @parent_object: parent #GMimeFilter
+ * @encode: encoding vs decoding line endings/dots
+ * @saw_cr: previous char was a CR
+ * @saw_ld: previous char was a LF
+ * @saw_dot: previous char was a period
  *
- * The direction in which the CRLF filter should convert.
+ * A filter to convert between line-ending formats and encode/decode
+ * lines beginning with a '.'.
  **/
-typedef enum {
-	GMIME_FILTER_CRLF_ENCODE,
-	GMIME_FILTER_CRLF_DECODE
-} GMimeFilterCRLFDirection;
-
-
-/**
- * GMimeFilterCRLFMode:
- * @GMIME_FILTER_CRLF_MODE_CRLF_DOTS: Escape lines beginning with a '.'
- * @GMIME_FILTER_CRLF_MODE_CRLF_ONLY: Do only LF->CRLF conversion
- *
- * The mode for the #GMimeFilterCRLF filter.
- **/
-typedef enum {
-	GMIME_FILTER_CRLF_MODE_CRLF_DOTS,
-	GMIME_FILTER_CRLF_MODE_CRLF_ONLY
-} GMimeFilterCRLFMode;
-
-
 struct _GMimeFilterCRLF {
 	GMimeFilter parent_object;
 	
-	GMimeFilterCRLFDirection direction;
-	GMimeFilterCRLFMode mode;
+	gboolean encode;
+	gboolean dots;
+	
 	gboolean saw_cr;
 	gboolean saw_lf;
 	gboolean saw_dot;
@@ -81,7 +66,7 @@ struct _GMimeFilterCRLFClass {
 
 GType g_mime_filter_crlf_get_type (void);
 
-GMimeFilter *g_mime_filter_crlf_new (GMimeFilterCRLFDirection direction, GMimeFilterCRLFMode mode);
+GMimeFilter *g_mime_filter_crlf_new (gboolean encode, gboolean dots);
 
 G_END_DECLS
 
