@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -26,6 +27,17 @@
 
 #include "gmime-filter-md5.h"
 
+
+/**
+ * SECTION: gmime-filter-md5
+ * @title: GMimeFilterMd5
+ * @short_description: Calculate an md5sum
+ * @see_also: #GMimeFilter
+ *
+ * Calculate an md5sum for a stream.
+ **/
+
+
 struct _GMimeFilterMd5Private {
 	MD5Context md5;
 };
@@ -117,7 +129,7 @@ filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 {
 	GMimeFilterMd5 *md5 = (GMimeFilterMd5 *) filter;
 	
-	md5_update (&md5->priv->md5, in, len);
+	md5_update (&md5->priv->md5, (unsigned char *) in, len);
 	
 	*out = in;
 	*outlen = len;
@@ -145,7 +157,7 @@ filter_reset (GMimeFilter *filter)
  *
  * Creates a new Md5 filter.
  *
- * Returns a new Md5 filter.
+ * Returns: a new Md5 filter.
  **/
 GMimeFilter *
 g_mime_filter_md5_new (void)
