@@ -48,7 +48,6 @@ static void g_mime_message_partial_init (GMimeMessagePartial *catpart, GMimeMess
 static void g_mime_message_partial_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void message_partial_init (GMimeObject *object);
 static void message_partial_prepend_header (GMimeObject *object, const char *header, const char *value);
 static void message_partial_append_header (GMimeObject *object, const char *header, const char *value);
 static void message_partial_set_header (GMimeObject *object, const char *header, const char *value);
@@ -95,7 +94,6 @@ g_mime_message_partial_class_init (GMimeMessagePartialClass *klass)
 	
 	gobject_class->finalize = g_mime_message_partial_finalize;
 	
-	object_class->init = message_partial_init;
 	object_class->prepend_header = message_partial_prepend_header;
 	object_class->append_header = message_partial_append_header;
 	object_class->remove_header = message_partial_remove_header;
@@ -122,13 +120,6 @@ g_mime_message_partial_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-message_partial_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
 static void
 message_partial_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
@@ -212,28 +203,29 @@ message_partial_set_content_type (GMimeObject *object, GMimeContentType *content
 GMimeMessagePartial *
 g_mime_message_partial_new (const char *id, int number, int total)
 {
+	GMimeContentType *content_type;
 	GMimeMessagePartial *partial;
-	GMimeContentType *type;
 	char *num;
 	
 	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL);
 	
-	type = g_mime_content_type_new ("message", "partial");
+	content_type = g_mime_content_type_new ("message", "partial");
 	
 	partial->id = g_strdup (id);
-	g_mime_content_type_set_parameter (type, "id", id);
+	g_mime_content_type_set_parameter (content_type, "id", id);
 	
 	partial->number = number;
 	num = g_strdup_printf ("%d", number);
-	g_mime_content_type_set_parameter (type, "number", num);
+	g_mime_content_type_set_parameter (content_type, "number", num);
 	g_free (num);
 	
 	partial->total = total;
 	num = g_strdup_printf ("%d", total);
-	g_mime_content_type_set_parameter (type, "total", num);
+	g_mime_content_type_set_parameter (content_type, "total", num);
 	g_free (num);
 	
-	g_mime_object_set_content_type (GMIME_OBJECT (partial), type);
+	g_mime_object_set_content_type (GMIME_OBJECT (partial), content_type);
+	g_object_unref (content_type);
 	
 	return partial;
 }
@@ -360,11 +352,9 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 		
 		wrapper = g_mime_part_get_content_object (GMIME_PART (partial));
 		stream = g_mime_data_wrapper_get_stream (wrapper);
-		g_object_unref (wrapper);
 		
 		g_mime_stream_reset (stream);
 		g_mime_stream_cat_add_source (GMIME_STREAM_CAT (cat), stream);
-		g_object_unref (stream);
 	}
 	
 	parser = g_mime_parser_new ();
@@ -390,21 +380,19 @@ message_partial_message_new (GMimeMessage *base)
 	const char *name, *value;
 	GMimeMessage *message;
 	GMimeHeaderList *list;
-	GMimeHeaderIter *iter;
+	GMimeHeaderIter iter;
 	
 	message = g_mime_message_new (FALSE);
 	
 	list = ((GMimeObject *) base)->headers;
-	iter = g_mime_header_list_get_iter (list);
-	if (g_mime_header_iter_is_valid (iter)) {
-		do {
-			name = g_mime_header_iter_get_name (iter);
-			value = g_mime_header_iter_get_value (iter);
-			g_mime_object_append_header ((GMimeObject *) message, name, value);
-		} while (g_mime_header_iter_next (iter));
-	}
 	
-	g_mime_header_iter_free (iter);
+	if (g_mime_header_list_get_iter (list, &iter)) {
+		do {
+			name = g_mime_header_iter_get_name (&iter);
+			value = g_mime_header_iter_get_value (&iter);
+			g_mime_object_append_header ((GMimeObject *) message, name, value);
+		} while (g_mime_header_iter_next (&iter));
+	}
 	
 	return message;
 }
