@@ -205,12 +205,13 @@ GMimeMultipartSigned *
 g_mime_multipart_signed_new (void)
 {
 	GMimeMultipartSigned *multipart;
-	GMimeContentType *type;
+	GMimeContentType *content_type;
 	
 	multipart = g_object_new (GMIME_TYPE_MULTIPART_SIGNED, NULL);
 	
-	type = g_mime_content_type_new ("multipart", "signed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
+	content_type = g_mime_content_type_new ("multipart", "signed");
+	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_object_unref (content_type);
 	
 	return multipart;
 }
@@ -241,11 +242,10 @@ sign_prepare (GMimeObject *mime_part)
 			return;
 		}
 		
-		n = g_mime_multipart_get_number (multipart);
+		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
 			subpart = g_mime_multipart_get_part (multipart, i);
 			sign_prepare (subpart);
-			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (mime_part)) {
 		subpart = GMIME_MESSAGE_PART (mime_part)->message->mime_part;
@@ -283,13 +283,13 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 			      GMimeCipherContext *ctx, const char *userid,
 			      GMimeCipherHash hash, GError **err)
 {
-	GMimeObject *signature;
-	GMimeDataWrapper *wrapper;
-	GMimeStream *filtered_stream;
-	GMimeStream *stream, *sigstream;
-	GMimeFilter *crlf_filter, *from_filter, *strip_filter;
+	GMimeStream *stream, *filtered, *sigstream;
 	GMimeContentType *content_type;
+	GMimeDataWrapper *wrapper;
+	GMimePart *signature;
+	GMimeFilter *filter;
 	GMimeParser *parser;
+	int rv;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), -1);
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), -1);
@@ -301,44 +301,51 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	
 	/* get the cleartext */
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new (stream);
+	filtered = g_mime_stream_filter_new (stream);
 	
 	/* Note: see rfc3156, section 3 - second note */
-	from_filter = g_mime_filter_from_new (GMIME_FILTER_FROM_MODE_ARMOR);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), from_filter);
-	g_object_unref (from_filter);
+	filter = g_mime_filter_from_new (GMIME_FILTER_FROM_MODE_ARMOR);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
 	/* Note: see rfc3156, section 5.4 (this is the main difference between rfc2015 and rfc3156) */
-	strip_filter = g_mime_filter_strip_new ();
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), strip_filter);
-	g_object_unref (strip_filter);
+	filter = g_mime_filter_strip_new ();
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
-	g_mime_object_write_to_stream (content, filtered_stream);
-	g_mime_stream_flush (filtered_stream);
-	g_object_unref (filtered_stream);
+	g_mime_object_write_to_stream (content, filtered);
+	g_mime_stream_flush (filtered);
+	g_object_unref (filtered);
 	g_mime_stream_reset (stream);
 	
 	/* Note: see rfc2015 or rfc3156, section 5.1 */
-	filtered_stream = g_mime_stream_filter_new (stream);
-	crlf_filter = g_mime_filter_crlf_new (TRUE, FALSE);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
-	g_object_unref (crlf_filter);
+	filtered = g_mime_stream_filter_new (stream);
+	filter = g_mime_filter_crlf_new (TRUE, FALSE);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
 	/* construct the signature stream */
 	sigstream = g_mime_stream_mem_new ();
 	
 	/* sign the content stream */
-	if (g_mime_cipher_context_sign (ctx, userid, hash, filtered_stream, sigstream, err) == -1) {
-		g_object_unref (filtered_stream);
+	if ((rv = g_mime_cipher_context_sign (ctx, userid, hash, filtered, sigstream, err)) == -1) {
 		g_object_unref (sigstream);
+		g_object_unref (filtered);
 		g_object_unref (stream);
 		return -1;
 	}
 	
-	g_object_unref (filtered_stream);
+	g_object_unref (filtered);
 	g_mime_stream_reset (sigstream);
 	g_mime_stream_reset (stream);
 	
+	/* set the multipart/signed protocol and micalg */
+	mps->protocol = g_strdup (ctx->sign_protocol);
+	mps->micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, (GMimeCipherHash) rv));
+	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "protocol", mps->protocol);
+	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "micalg", mps->micalg);
+	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
+	
 	/* construct the content part */
 	parser = g_mime_parser_new_with_stream (stream);
 	content = g_mime_parser_construct_part (parser);
@@ -346,40 +353,31 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_object_unref (parser);
 	
 	/* construct the signature part */
-	signature = (GMimeObject *) g_mime_part_new ();
+	content_type = g_mime_content_type_new_from_string (mps->protocol);
+	signature = g_mime_part_new_with_type (content_type->type, content_type->subtype);
+	g_object_unref (content_type);
+	
 	wrapper = g_mime_data_wrapper_new ();
 	g_mime_data_wrapper_set_stream (wrapper, sigstream);
-	g_mime_part_set_content_object (GMIME_PART (signature), wrapper);
+	g_mime_part_set_content_object (signature, wrapper);
 	g_object_unref (sigstream);
 	g_object_unref (wrapper);
 	
-	mps->protocol = g_strdup (ctx->sign_protocol);
-	mps->micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, hash));
-	
-	/* set the content-type of the signature part */
-	content_type = g_mime_content_type_new_from_string (mps->protocol);
-	g_mime_object_set_content_type (signature, content_type);
-	
 	/* FIXME: temporary hack, this info should probably be set in
 	 * the CipherContext class - maybe ::sign can take/output a
 	 * GMimePart instead. */
 	if (!g_ascii_strcasecmp (mps->protocol, "application/pkcs7-signature")) {
-		g_mime_part_set_content_encoding (GMIME_PART (signature), GMIME_CONTENT_ENCODING_BASE64);
-		g_mime_part_set_filename (GMIME_PART (signature), "smime.p7m");
+		g_mime_part_set_content_encoding (signature, GMIME_CONTENT_ENCODING_BASE64);
+		g_mime_part_set_filename (signature, "smime.p7m");
 	}
 	
 	/* save the content and signature parts */
 	/* FIXME: make sure there aren't any other parts?? */
-	g_mime_multipart_add_part (GMIME_MULTIPART (mps), content);
-	g_mime_multipart_add_part (GMIME_MULTIPART (mps), signature);
+	g_mime_multipart_add (GMIME_MULTIPART (mps), content);
+	g_mime_multipart_add (GMIME_MULTIPART (mps), (GMimeObject *) signature);
 	g_object_unref (signature);
 	g_object_unref (content);
 	
-	/* set the content-type params for this multipart/signed part */
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "micalg", mps->micalg);
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "protocol", mps->protocol);
-	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
-	
 	return 0;
 }
 
@@ -415,7 +413,7 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (ctx->sign_protocol != NULL, NULL);
 	
-	if (g_mime_multipart_get_number ((GMimeMultipart *) mps) < 2) {
+	if (g_mime_multipart_get_count ((GMimeMultipart *) mps) < 2) {
 		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
 			     "Cannot verify multipart/signed part due to missing subparts.");
 		return NULL;
@@ -438,7 +436,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_object_unref (signature);
 		g_free (content_type);
 		
 		return NULL;
@@ -460,7 +457,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_mime_stream_flush (filtered_stream);
 	g_object_unref (filtered_stream);
 	g_mime_stream_reset (stream);
-	g_object_unref (content);
 	
 	/* get the signature stream */
 	wrapper = g_mime_part_get_content_object (GMIME_PART (signature));
@@ -477,8 +473,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	}
 	
 	g_mime_stream_reset (sigstream);
-	g_object_unref (signature);
-	g_object_unref (wrapper);
 	
 	/* verify the signature */
 	hash = g_mime_cipher_context_hash_id (ctx, mps->micalg);
@@ -487,7 +481,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	d(printf ("attempted to verify:\n----- BEGIN SIGNED PART -----\n%.*s----- END SIGNED PART -----\n",
 		  (int) GMIME_STREAM_MEM (stream)->buffer->len, GMIME_STREAM_MEM (stream)->buffer->data));
 	
-	g_object_unref (sigstream);
 	g_object_unref (stream);
 	
 	return valid;
