@@ -42,12 +42,22 @@ G_BEGIN_DECLS
 typedef struct _GMimeObject GMimeObject;
 typedef struct _GMimeObjectClass GMimeObjectClass;
 
+/**
+ * GMimeObject:
+ * @parent_object: parent #GObject
+ * @disposition: a #GMimeContentDisposition
+ * @content_type: a #GMimeContentType
+ * @content_id: a Content-Id
+ * @headers: a #GMimeHeaderList
+ *
+ * Base class for all MIME parts.
+ **/
 struct _GMimeObject {
 	GObject parent_object;
 	
 	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
-	GMimeHeader *headers;
+	GMimeHeaderList *headers;
 	
 	char *content_id;
 };
@@ -55,12 +65,11 @@ struct _GMimeObject {
 struct _GMimeObjectClass {
 	GObjectClass parent_class;
 	
-	void         (* init)          (GMimeObject *object);
-	
-	void         (* add_header)    (GMimeObject *object, const char *header, const char *value);
-	void         (* set_header)    (GMimeObject *object, const char *header, const char *value);
-	const char * (* get_header)    (GMimeObject *object, const char *header);
-	void         (* remove_header) (GMimeObject *object, const char *header);
+	void         (* prepend_header) (GMimeObject *object, const char *header, const char *value);
+	void         (* append_header)  (GMimeObject *object, const char *header, const char *value);
+	void         (* set_header)     (GMimeObject *object, const char *header, const char *value);
+	const char * (* get_header)     (GMimeObject *object, const char *header);
+	gboolean     (* remove_header)  (GMimeObject *object, const char *header);
 	
 	void         (* set_content_type) (GMimeObject *object, GMimeContentType *content_type);
 	
@@ -84,15 +93,17 @@ typedef void (* GMimePartFunc) (GMimeObject *part, gpointer user_data);
 GType g_mime_object_get_type (void);
 
 void g_mime_object_register_type (const char *type, const char *subtype, GType object_type);
+
+GMimeObject *g_mime_object_new (GMimeContentType *content_type);
 GMimeObject *g_mime_object_new_type (const char *type, const char *subtype);
 
 void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type);
-const GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
+GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
 void g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name, const char *value);
 const char *g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name);
 
 void g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition);
-const GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *object);
+GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *object);
 
 void g_mime_object_set_disposition (GMimeObject *object, const char *disposition);
 const char *g_mime_object_get_disposition (GMimeObject *object);
@@ -103,10 +114,13 @@ const char *g_mime_object_get_content_disposition_parameter (GMimeObject *object
 void g_mime_object_set_content_id (GMimeObject *object, const char *content_id);
 const char *g_mime_object_get_content_id (GMimeObject *object);
 
-void g_mime_object_add_header (GMimeObject *object, const char *header, const char *value);
+void g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value);
+void g_mime_object_append_header (GMimeObject *object, const char *header, const char *value);
 void g_mime_object_set_header (GMimeObject *object, const char *header, const char *value);
 const char *g_mime_object_get_header (GMimeObject *object, const char *header);
-void g_mime_object_remove_header (GMimeObject *object, const char *header);
+gboolean g_mime_object_remove_header (GMimeObject *object, const char *header);
+
+GMimeHeaderList *g_mime_object_get_header_list (GMimeObject *object);
 
 char *g_mime_object_get_headers (GMimeObject *object);
 
