@@ -317,13 +317,13 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->from_offset = -1;
 	priv->from_line = g_byte_array_new ();
 	
-	priv->headerbuf = g_malloc (SCAN_HEAD + 1);
+	priv->headerbuf = g_malloc (SCAN_HEAD);
 	priv->headerptr = priv->headerbuf;
-	priv->headerleft = SCAN_HEAD;
+	priv->headerleft = SCAN_HEAD - 1;
 	
-	priv->rawbuf = g_malloc (SCAN_HEAD + 1);
+	priv->rawbuf = g_malloc (SCAN_HEAD);
 	priv->rawptr = priv->rawbuf;
-	priv->rawleft = SCAN_HEAD;
+	priv->rawleft = SCAN_HEAD - 1;
 	
 	priv->headers_start = -1;
 	priv->header_start = -1;
@@ -361,7 +361,7 @@ parser_close (GMimeParser *parser)
  *
  * Creates a new parser object.
  *
- * Returns a new parser object.
+ * Returns: a new parser object.
  **/
 GMimeParser *
 g_mime_parser_new (void)
@@ -380,7 +380,7 @@ g_mime_parser_new (void)
  *
  * Creates a new parser object preset to parse @stream.
  *
- * Returns a new parser object.
+ * Returns: a new parser object.
  **/
 GMimeParser *
 g_mime_parser_new_with_stream (GMimeStream *stream)
@@ -432,7 +432,7 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
  *
  * Gets whether or not the underlying stream is persistent.
  *
- * Returns %TRUE if the @parser will leave the content on disk or
+ * Returns: %TRUE if the @parser will leave the content on disk or
  * %FALSE if it will load the content into memory.
  **/
 gboolean
@@ -477,7 +477,7 @@ g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
  *
  * Gets whether or not @parser is set to scan mbox-style From-lines.
  *
- * Returns whether or not @parser is set to scan mbox-style
+ * Returns: whether or not @parser is set to scan mbox-style
  * From-lines.
  **/
 gboolean
@@ -512,7 +512,7 @@ g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
  * Gets whether or not @parser is set to use Content-Length for
  * determining the offset of the end of the message.
  *
- * Returns whether or not @parser is set to use Content-Length for
+ * Returns: whether or not @parser is set to use Content-Length for
  * determining the offset of the end of the message.
  **/
 gboolean
@@ -646,7 +646,7 @@ parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
  *
  * Gets the current stream offset from the parser's internal stream.
  *
- * Returns the current stream offset from the parser's internal stream
+ * Returns: the current stream offset from the parser's internal stream
  * or -1 on error.
  **/
 gint64
@@ -665,7 +665,7 @@ g_mime_parser_tell (GMimeParser *parser)
  *
  * Tests the end-of-stream indicator for @parser's internal stream.
  *
- * Returns %TRUE on EOS or %FALSE otherwise.
+ * Returns: %TRUE on EOS or %FALSE otherwise.
  **/
 gboolean
 g_mime_parser_eos (GMimeParser *parser)
@@ -741,19 +741,31 @@ parser_step_from (GMimeParser *parser)
 	return 0;
 }
 
+static size_t
+nearest_pow (size_t num)
+{
+	size_t n = num > 0 ? num - 1 : 0;
+	
+	n |= n >> 1;
+	n |= n >> 2;
+	n |= n >> 4;
+	n |= n >> 8;
+	n |= n >> 16;
+	n++;
+	
+	return n;
+}
+
 #define header_append(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
 		size_t hlen, hoff;                                        \
 		                                                          \
-		hlen = hoff = priv->headerptr - priv->headerbuf;          \
-		hlen = hlen ? hlen : 1;                                   \
+		hoff = priv->headerptr - priv->headerbuf;                 \
+		hlen = nearest_pow (hoff + len + 1);                      \
 		                                                          \
-		while (hlen < hoff + len)                                 \
-			hlen <<= 1;                                       \
-		                                                          \
-		priv->headerbuf = g_realloc (priv->headerbuf, hlen + 1);  \
+		priv->headerbuf = g_realloc (priv->headerbuf, hlen);      \
 		priv->headerptr = priv->headerbuf + hoff;                 \
-		priv->headerleft = hlen - hoff;                           \
+		priv->headerleft = (hlen - 1) - hoff;                     \
 	}                                                                 \
 	                                                                  \
 	memcpy (priv->headerptr, start, len);                             \
@@ -765,15 +777,12 @@ parser_step_from (GMimeParser *parser)
 	if (priv->rawleft <= len) {                                       \
 		size_t hlen, hoff;                                        \
 		                                                          \
-		hlen = hoff = priv->rawptr - priv->rawbuf;                \
-		hlen = hlen ? hlen : 1;                                   \
+		hoff = priv->rawptr - priv->rawbuf;                       \
+		hlen = nearest_pow (hoff + len + 1);                      \
 		                                                          \
-		while (hlen < hoff + len)                                 \
-			hlen <<= 1;                                       \
-		                                                          \
-		priv->rawbuf = g_realloc (priv->rawbuf, hlen + 1);        \
+		priv->rawbuf = g_realloc (priv->rawbuf, hlen);            \
 		priv->rawptr = priv->rawbuf + hoff;                       \
-		priv->rawleft = hlen - hoff;                              \
+		priv->rawleft = (hlen - 1) - hoff;                        \
 	}                                                                 \
 	                                                                  \
 	memcpy (priv->rawptr, start, len);                                \
@@ -1330,7 +1339,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
+		g_mime_object_append_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1361,17 +1370,16 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (object, header->name, header->value);
+		g_mime_object_append_header (object, header->name, header->value);
 		header = header->next;
 	}
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
+	g_mime_object_set_content_type (object, content_type);
+	g_object_unref (content_type);
 	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
 	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
@@ -1491,7 +1499,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		else
 			subpart = parser_construct_leaf_part (parser, content_type, &found);
 		
-		g_mime_multipart_add_part (multipart, subpart);
+		g_mime_multipart_add (multipart, subpart);
 		g_object_unref (subpart);
 	} while (found == FOUND_BOUNDARY && found_immediate_boundary (priv, FALSE));
 	
@@ -1513,17 +1521,16 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (object, header->name, header->value);
+		g_mime_object_append_header (object, header->name, header->value);
 		header = header->next;
 	}
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
+	g_mime_object_set_content_type (object, content_type);
+	g_object_unref (content_type);
 	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
 	multipart = (GMimeMultipart *) object;
@@ -1594,7 +1601,7 @@ parser_construct_part (GMimeParser *parser)
  *
  * Constructs a MIME part from @parser.
  *
- * Returns a MIME part based on @parser or %NULL on fail.
+ * Returns: a MIME part based on @parser or %NULL on fail.
  **/
 GMimeObject *
 g_mime_parser_construct_part (GMimeParser *parser)
@@ -1638,7 +1645,7 @@ parser_construct_message (GMimeParser *parser)
 				content_length = ULONG_MAX;
 		}
 		
-		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
+		g_mime_object_append_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1673,7 +1680,7 @@ parser_construct_message (GMimeParser *parser)
  *
  * Constructs a MIME message from @parser.
  *
- * Returns a MIME message or %NULL on fail.
+ * Returns: a MIME message or %NULL on fail.
  **/
 GMimeMessage *
 g_mime_parser_construct_message (GMimeParser *parser)
@@ -1691,7 +1698,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
  * Gets the mbox-style From-line of the most recently parsed message
  * (gotten from g_mime_parser_construct_message()).
  *
- * Returns the mbox-style From-line of the most recently parsed
+ * Returns: the mbox-style From-line of the most recently parsed
  * message or %NULL on error.
  **/
 char *
@@ -1719,7 +1726,7 @@ g_mime_parser_get_from (GMimeParser *parser)
  * Gets the offset of the most recently parsed mbox-style From-line
  * (gotten from g_mime_parser_construct_message()).
  *
- * Returns the offset of the most recently parsed mbox-style From-line
+ * Returns: the offset of the most recently parsed mbox-style From-line
  * or -1 on error.
  **/
 gint64
