@@ -362,7 +362,6 @@ write_content (GMimePart *part, GMimeStream *stream)
 		content_stream = g_mime_data_wrapper_get_stream (part->content);
 		g_mime_stream_reset (content_stream);
 		nwritten = g_mime_stream_write_to_stream (content_stream, stream);
-		g_object_unref (content_stream);
 		
 		if (nwritten == -1)
 			return -1;
@@ -412,13 +411,14 @@ mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 GMimePart *
 g_mime_part_new (void)
 {
+	GMimeContentType *content_type;
 	GMimePart *mime_part;
-	GMimeContentType *type;
 	
 	mime_part = g_object_new (GMIME_TYPE_PART, NULL);
 	
-	type = g_mime_content_type_new ("text", "plain");
-	g_mime_object_set_content_type (GMIME_OBJECT (mime_part), type);
+	content_type = g_mime_content_type_new ("text", "plain");
+	g_mime_object_set_content_type (GMIME_OBJECT (mime_part), content_type);
+	g_object_unref (content_type);
 	
 	return mime_part;
 }
@@ -436,56 +436,19 @@ g_mime_part_new (void)
 GMimePart *
 g_mime_part_new_with_type (const char *type, const char *subtype)
 {
-	GMimePart *mime_part;
 	GMimeContentType *content_type;
+	GMimePart *mime_part;
 	
 	mime_part = g_object_new (GMIME_TYPE_PART, NULL);
 	
 	content_type = g_mime_content_type_new (type, subtype);
 	g_mime_object_set_content_type (GMIME_OBJECT (mime_part), content_type);
+	g_object_unref (content_type);
 	
 	return mime_part;
 }
 
 
-/**
- * g_mime_part_set_content_header:
- * @mime_part: mime part
- * @header: header name
- * @value: header value
- *
- * Set an arbitrary MIME content header.
- **/
-void
-g_mime_part_set_content_header (GMimePart *mime_part, const char *header, const char *value)
-{
-	g_return_if_fail (GMIME_IS_PART (mime_part));
-	g_return_if_fail (header != NULL);
-	
-	g_mime_object_set_header (GMIME_OBJECT (mime_part), header, value);
-}
-
-
-/**
- * g_mime_part_get_content_header:
- * @mime_part: mime part
- * @header: header name
- *
- * Gets the value of the requested header if it exists, or %NULL
- * otherwise.
- *
- * Returns: the value of the content header @header.
- **/
-const char *
-g_mime_part_get_content_header (GMimePart *mime_part, const char *header)
-{
-	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
-	g_return_val_if_fail (header != NULL, NULL);
-	
-	return g_mime_object_get_header (GMIME_OBJECT (mime_part), header);
-}
-
-
 /**
  * g_mime_part_set_content_description:
  * @mime_part: Mime part
@@ -569,8 +532,8 @@ void
 g_mime_part_set_content_md5 (GMimePart *mime_part, const char *content_md5)
 {
 	unsigned char digest[16], b64digest[32];
-	const GMimeContentType *content_type;
 	GMimeStreamFilter *filtered_stream;
+	GMimeContentType *content_type;
 	GMimeFilter *md5_filter;
 	GMimeStream *stream;
 	guint32 save = 0;
@@ -633,8 +596,8 @@ gboolean
 g_mime_part_verify_content_md5 (GMimePart *mime_part)
 {
 	unsigned char digest[16], b64digest[32];
-	const GMimeContentType *content_type;
 	GMimeStreamFilter *filtered_stream;
+	GMimeContentType *content_type;
 	GMimeFilter *md5_filter;
 	GMimeStream *stream;
 	guint32 save = 0;
@@ -785,9 +748,6 @@ g_mime_part_set_filename (GMimePart *mime_part, const char *filename)
 	
 	g_return_if_fail (GMIME_IS_PART (mime_part));
 	
-	if (!object->disposition)
-		g_mime_object_set_disposition (object, GMIME_DISPOSITION_ATTACHMENT);
-	
 	g_mime_object_set_content_disposition_parameter (object, "filename", filename);
 	g_mime_object_set_content_type_parameter (object, "name", filename);
 }
@@ -856,8 +816,5 @@ g_mime_part_get_content_object (const GMimePart *mime_part)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
 	
-	if (mime_part->content)
-		g_object_ref (mime_part->content);
-	
 	return mime_part->content;
 }
